#!/usr/bin/perl

$VER = '0.1.2';

## Add Certificate to DB - Version $VER
## (c) 1999 by Massimiliano Pala 
## OpenCA Core Team
##
## (OpenCA Project)
##
## Description:
## ============
##
## This program adds a certificates to the DB where the
## ca program will look for certificates. 
## This script will be run when you issue certificate through the
## scritps in the bin/ directory and not using the Web interface.
##
## The DB structure is very simple, every entry looks like
## the following :
##
##	$serial => $certificate
##
## Please take note that this program, if the DB is not present
## will create it: you must be sure you run it as the user the
## httpd server will run (as the ca program needs to access to
## the DB to properly work). If permissions are not set, please
## change them or rebuild the DB (delete it and use this prog.)
##
## For Any clarifications, please refer to:
##
##	o Documentation;
##	o openca-devel@openca.org;
##

my $fileName = $ARGV[0];
my $cert = "";
my $OPENCA_DIR = $ENV{'ca'};
my $SSL_DIR = $ENV{'ssl'};

if( "$OPENCA_DIR" eq "" ) {
	$OPENCA_DIR = '/usr/local/OpenCA';
}

if( "$SSL_DIR" eq "" ) {
	$SSL_DIR = "/usr/local/ssl";
}

$DBM_FILE = "$OPENCA_DIR/stuff/issued_certs";
$STATUS_FILE = "$OPENCA_DIR/stuff/revoked_certs";


if ( "$fileName" eq "" ) {
	print "ERROR: Certificate filename needed.\n";
	exit 1;
}

$info = `$SSL_DIR/bin/openssl x509 -text -noout <$fileName`;

if ( "$info" =~ /error/i ) {
	print "ERROR: Can not load certificate ($fileName).\n";
	exit 1;
}

( $serial ) =
	( $info =~ /Serial Number:[^x]*.([^\)]+)/i );

( $notBefore ) =
 	( $info =~ /Not Before: ([^\n]+)/i );

( $notAfter ) =
	( $info =~ /Not after : ([^\n]+)/i );

if ( length( $serial ) < 2 ) {
 	$serial = "0" . uc($serial);
}

open( FD, "$fileName" ) or die "ERROR: Can't access $fileName\n\n";
	while( $tmp = <FD> ) {
		next if ( ($cert eq "") and ($tmp !~ /BEGIN CERTIFICATE/ ));
		$cert .= $tmp;
	}
close( FD );

dbmopen( %CERT_DB, "$DBM_FILE", 0600 ) || die "Can't access DB!";
	$CERT_DB{$serial} = "$cert";
dbmclose( %CERT_DB );

exit 0;
